@@ -1,6 +1,8 @@
+__version__ = '0.0.3'
+
 from gym.envs.registration import register
 
 register(
     id='CRN-v0',
-    entry_point='cybergenetics.envs:CRNEnv',
+    entry_point='cybergenetics.envs.crn:CRNWrapper',
 )
